# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

newproperty(:channel) do
  include EasyType
  include PuppetX::EnterpriseModules::ExtendHash
  include PuppetX::EnterpriseModules::Oracle::Access

  desc <<-DESC
    Manage rman configuration for `CHANNEL` used to configure specific (numbered) channels.
    Using the CONNECT operand is not supported (yet).
    The configuration can be set to default by setting the specific channel to 'clear' (case insensitive).

    ora_rman_config { 'SID':
      ...
      channel => {
        'DISK' => {
          'DEFAULT' => 'MAXPIECESIZE 1024M RATE 1G',
          '1'       => "FORMAT '/u01/backup/SID_%U' MAXPIECESIZE 1024M RATE 1G",
          '2'       => 'CLEAR',
        },
        'SBT' => {
          '1'       => 'MAXPIECESIZE 1024M RATE 1G',
          '2'       => 'CLEAR',
          'DEFAULT' => 'MAXPIECESIZE 2048 M RATE 2G',
        },
      },
      ...
    }
  DESC

  data_type("Optional[Hash[Enum['DISK','SBT'],Hash[String[1],String[1]]]]")

  to_translate_to_resource do |raw_resource|
    raw_resource['CHANNEL']
  end

  def insync?(is)
    return true if is == should

    checkdiff(is, should).empty?
  end

  def change_to_s(currentvalue, newvalue)
    diff = checkdiff(currentvalue, newvalue)
    messages = []
    diff.each_pair do |type, val|
      val.each_pair do |channel, properties|
        messages << if channel.casecmp('default').zero?
                      if properties.casecmp('clear').zero?
                        "clearing channel for device type #{type}"
                      else
                        "changing channel for device type #{type} to #{properties}"
                      end
                    elsif properties.casecmp('clear').zero?
                      "clearing channel #{channel} for device type #{type}"
                    else
                      "changing channel #{channel} for device type #{type} to #{properties}"
                    end
      end
    end
    messages.join(', ')
  end

  on_modify do
    diff = checkdiff(provider.channel, should)
    diff.each_pair do |channel_type, values|
      values.each_pair do |channel, channel_value|
        if channel.casecmp('default').zero?
          rman("configure channel device type #{channel_type} #{channel_value}", { :sid => resource.sid, :target_conn => resource.target_conn, :catalog_conn => resource.catalog_conn })
        else
          rman("configure channel #{channel} device type #{channel_type} #{channel_value}", { :sid => resource.sid, :target_conn => resource.target_conn, :catalog_conn => resource.catalog_conn })
        end
      end
    end
  end

  def checkdiff(is, should)
    my_is = if is.to_s == 'absent'
              {}
            else
              is.collect { |type| Hash(type[0].upcase => type[1].transform_values { |value| value.split.reduce('') { |mem, word| word.scan(/^\d+$/).any? ? "#{mem} #{word}" : "#{mem}#{word} " }.gsub('  ', ' ').strip }) }.reduce({}, :merge)
            end
    my_should = should.collect { |type| Hash(type[0].upcase => type[1].transform_values { |value| value.split.reduce('') { |mem, word| word.scan(/^\d+$/).any? ? "#{mem} #{word}" : "#{mem}#{word} " }.gsub('  ', ' ').strip }) }.reduce({}, :merge)

    my_should.each_pair do |type, values|
      values.each_pair do |channel, properties|
        my_should[type].delete(channel) if properties.casecmp('clear').zero? && !my_is.dig(type, channel)
        my_should[type].delete(channel) if my_is.dig(type, channel) == properties
      end
    end
    my_should.delete_if { |_type, values| values.empty? }
  end
end
