# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    module Oracle
      # Docs
      module GrantProperty
        def self.included(parent)
          parent.extend(ClassMethods)
        end

        def change_to_s(_from, _to)
          return_value = []
          return_value << "revoked the #{revoked_rights.join(',')} #{admin_option} right(s)" if revoked_rights.any?
          return_value << "granted the #{granted_rights.join(',')} #{admin_option} right(s)" if granted_rights.any?
          return_value.join(' and ')
        end

        def revoked_rights
          []
        end

        def granted_rights
          []
        end

        def current_rights
          rights_for_grantee(resource.grant_key, resource.sid)
        end

        def revoke(right)
          "revoke #{right} from #{resource.grant_key} #{common_option}"
        end

        def grant(rights)
          "grant #{rights} to #{resource.grant_key} #{admin_option} #{common_option}"
        end

        def rights_for_grantee(user, sid)
          self.class.rights_for_grantee(user, sid)
        end

        def after_apply
          granted_rights.each do |right|
            sql(grant(right), :sid => resource.sid)
          end
          revoked_rights.each do |right|
            sql(revoke(right), :sid => resource.sid)
          end
        end

        def admin_option
          self.class.admin ? 'with admin option' : ''
        end

        def common_option
          #
          # When you use this clause on a non container database, warn.
          #
          return 'container = all' if containerdb?(resource.sid) && self.class.common

          Puppet.warn_once('warnings', resource.path, "#{resource.path}: Ignoring container clause of non-container database.") if self.class.common
          ''
        end

        # Docs
        module ClassMethods
          def translate_to_resource(raw_resource)
            @all_rights ||= privileges + granted_roles
            grantee     = raw_resource.column_data(grantee_column).upcase
            sid         = raw_resource.column_data('SID')
            rights_for_grantee(grantee, sid)
          end

          #
          # Proxy to the type class. So we only have to specify it once for all
          # grant related properties in a type
          #
          def grantee_column
            to_s.split('::')[0..-2].reduce(Kernel) { |root, new| root.const_get(new) }.grantee_column
          end

          def rights_for_grantee(grantee, sid)
            @all_rights.select { |r| r['GRANTEE'] == grantee && r['SID'] == sid }.collect { |u| u['PRIVILEGE'] }
          end

          def admin_option
            admin ? 'YES' : 'NO'
          end

          def common_option
            common ? 'YES' : 'NO'
          end

          def privileges
            #
            # For older Oracle versions we must hack arround the common option. So
            # when the Oracle version does not support containers, and we are a common
            # grant, we return use a query that returns an empty result.
            #
            no_container_query = if common
                                   "select * from dual where dummy='ORA'"
                                 else
                                   %(select *
                                       from (select distinct grantee
                                                           , privilege
                                                           , admin_option
                                               from dba_sys_privs
                                              union
                                             select username as grantee
                                                  , 'SYSDBA' as privilege
                                                  , 'NO' as admin_option
                                               from v$pwfile_users
                                              where sysdba = 'TRUE')
                                      where admin_option = '#{admin_option}')
                                 end
            query = for_version(
              :'11' => no_container_query,
              /12\.1.*/ => no_container_query,
              :default => %(select *
                              from (select distinct grantee
                                                  , privilege
                                                  , admin_option
                                                  , common
                                      from dba_sys_privs
                                     union
                                    select username as grantee
                                         , 'SYSDBA' as privilege
                                         , 'NO' as admin_option
                                         , common
                                      from v$pwfile_users
                                     where sysdba = 'TRUE')
                             where admin_option = '#{admin_option}'
                               and common = '#{common_option}')
            )
            sql_on_all_primary_database_sids query
          end

          def granted_roles
            #
            # For older Oracle versions we must hack arround the common option. So
            # when the Oracle version does not support containers, and we are a common
            # grant, we return use a query that returns an empty result.
            #
            no_container_query = if common
                                   "select * from dual where dummy='ORA'"
                                 else
                                   "select distinct grantee, granted_role as privilege from dba_role_privs where admin_option = '#{admin_option}'"
                                 end
            query = for_version(
              :'11' => no_container_query,
              /12\.1.*/ => no_container_query,
              :default => "select distinct grantee, granted_role as privilege from dba_role_privs where admin_option = '#{admin_option}' and common = '#{common_option}'"
            )
            sql_on_all_primary_database_sids query
          end
        end
      end
    end
  end
end
