# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/oracle/command'

module PuppetX
  module EnterpriseModules
    module Oracle
      # docs
      class RmanCommand < PuppetX::EnterpriseModules::Oracle::Command
        include ::EasyType::Template
        include Settings
        include ::EasyType::Encryption

        RMAN_VALID_OPTIONS ||= [
          :sid,
          :target_conn,
          :catalog_conn
        ].freeze

        attr_reader :sid, :target_conn, :catalog_conn

        def initialize(options = {})
          target = options[:target_conn].nil? ? 'target /' : "target #{options[:target_conn]}"
          catalog = options[:catalog_conn].nil? ? 'nocatalog' : "catalog #{options[:catalog_conn]}"
          super("rman #{target} #{catalog}", options, RMAN_VALID_OPTIONS)
          #
          # Set only values that are needed
          #
          # rubocop:disable Lint/UselessAssignment
          os_user                 = configuration_value_for(@sid, 'os_user')
          # rubocop:enable Lint/UselessAssignment
          @oracle_home            = configuration_value_for(@sid, 'oracle_home')
          @contained_by           = configuration_value_for(@sid, 'contained_by')
          @host_sid = if @contained_by.nil? || @contained_by.empty?
                        @sid
                      else
                        @contained_by
                      end
        end

        def command_string(arguments = '')
          "su - #{@os_user} -c \"cd #{working_dir};export PERL5LIB=$(ls -d #{@oracle_home}/perl/lib/[0-9]*);export NLS_LANG=english;export ORACLE_SID=#{@host_sid};export ORACLE_HOME=#{@oracle_home};unset TNS_ADMIN;unset TWO_TASK;export PATH=#{@oracle_home}/bin:$PATH; #{@command} <<-EOF
            #{arguments};
EOF\""
        end

        def execute(arguments, powershell_script = '')
          options = { :failonfail => true, :combine => true }
          command = if Puppet::Util::Platform.windows?
                      %(powershell -File "#{powershell_script}")
                    else
                      command_string(arguments)
                    end
          Puppet.debug "rman_command.rb: Executing #{@command} command: #{arguments} as user #{os_user}"
          value = Puppet::Util::Execution.execute(command, options)
          Puppet.debug "rman_command.rb: Output:\n #{value}"
          value.encode!('UTF-8', :invalid => :replace, :undef => :replace)

          # Strip 'header' and 'footer'
          value[/\n\nRMAN> \n(.*)\nRMAN> /m, 1]
        end

        def execute_sql_command(command, output_file)
          #
          # Do the stuff
          #
          Puppet.debug "rman_command.rb: Executing sql statement :\n #{command}"
          script = command_file(template('ora_config/ora_config/execute.sql.erb', binding))
          if Puppet::Util::Platform.windows?
            # rubocop:disable Lint/UselessAssignment
            powershell_script = command_file(template('ora_config/ora_config/rman_command.ps1.erb', binding), type = '.ps1')
            # rubocop:enable Lint/UselessAssignment
            Puppet.debug "rman_command.rb: Using powershell script: #{powershell_script}"
          end
          if username
            Puppet.debug "rman_command.rb: Connecting to oracle_sid #{@sid} with connect_string \"#{@connect_string}\" as user #{username} and privilege #{@syspriv}"
          else
            Puppet.debug "rman_command.rb: Connecting to oracle_sid #{@sid} with connect_string \"#{@connect_string}\" as user #{@sys_username} and privilege #{@syspriv}"
          end
          execute("@#{script}", powershell_script)
          content = File.read(output_file)
          content.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          content
        end
      end
    end
  end
end
